#!/bin/sh

LC_ALL=C singularity exec \{% if pkg.singularity_args %}
  {{ pkg.singularity_args }} \{% endif %}{% for volume in pkg.volumes %}{% if volume != "$HOME"  and volume != "${HOME}" %}
  -B "{{volume}}:{{volume}}" \{% endif %}{% endfor %}
  {{ pkg.singularity_fullpath }}{% if pkg.docker_command %} {{ pkg.docker_command }}{% elif pkg.command %} {{ pkg.command }}{% endif %} "$@"
